﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace Anapico_CSharp_Example
{
    public class ApComm
    {
        public const int IFACE_LAN = 0x00000001;
        public const int IFACE_USB = 0x00000010;
        public const int IFACE_VISA_GPIB = 0x00000200;

        // DLL name, "ap_comm_lib.dll" for x86, "ap_comm_lib_64.dll" for x64 
        //public const string DLL_NAME = "ap_comm_lib.dll";
        public const string DLL_NAME = "ap_comm_lib_64.dll";

        // DLL functions
        // Not all functions implemented, only those needed for basic communication.
        // Please see the ap_comm_lib.h file for functions provided by the Anapico Communication Library.
        // More functions can be added easily, similar to the functions below.
        [DllImport(DLL_NAME, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ap_comm_find(IntPtr devices, IntPtr tmoMs, IntPtr count, IntPtr retCount, IntPtr iface);
        [DllImport(DLL_NAME, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ap_comm_open(IntPtr address, IntPtr tmoMs, IntPtr retLinkId);
        [DllImport(DLL_NAME, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ap_comm_close(IntPtr linkId);
        [DllImport(DLL_NAME, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ap_comm_puts(IntPtr linkId, IntPtr cmd, IntPtr count, IntPtr retCount);
        [DllImport(DLL_NAME, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ap_comm_gets(IntPtr linkId, IntPtr ans, IntPtr count, IntPtr retCount);
    }
    class Program
    {
        static void Main(string[] args)
        {
            // Communication buffer
            const int bufferSize = 1024;
            var vAns = Marshal.AllocHGlobal(bufferSize);
            // Communication timeout in milliseconds (recommended 2000 ms)
            const int tmoMs = 2000;
            // Communication status
            int status = 0;

            Console.WriteLine("Anapico C# example, DLL used: " + ApComm.DLL_NAME);

            // Find and select device (optional)
            status = ApComm.ap_comm_find(vAns, new IntPtr(2000), new IntPtr(bufferSize), IntPtr.Zero, new IntPtr(ApComm.IFACE_LAN | ApComm.IFACE_USB));
            string sDevices = Marshal.PtrToStringAnsi(vAns);
            string[] devices = sDevices.Split(';');
            for (int i = 0; i < devices.Length - 1; i++)
            {
                Console.WriteLine(i + ": " + devices[i]);
                //Console.WriteLine(devices[i]);
            }
            Console.Write("Select device: ");
            string device = devices[int.Parse(Console.ReadLine())].Split(':')[2];

            // Device address if device-find-and-select not used
            //string device = "192.168.1.81";

            // Open connection
            Console.WriteLine("Opening connection to " + device);
            var vDevice = Marshal.StringToHGlobalAnsi(device);
            var vLinkId = Marshal.AllocHGlobal(IntPtr.Size);
            status = ApComm.ap_comm_open(vDevice, new IntPtr(tmoMs), vLinkId);
            //System.int
            IntPtr linkId = Marshal.ReadIntPtr(vLinkId);
            //int linkId = 0;
            Console.WriteLine("    done, link ID " + linkId + ", status " + status + ".");

            // Identify device (*IDN query)
            Console.WriteLine("Sending *IDN? query");
            status = ApComm.ap_comm_puts(linkId, Marshal.StringToHGlobalAnsi("*IDN?"), IntPtr.Zero, IntPtr.Zero);
            Console.WriteLine("    done, status " + status + ".");
            Console.WriteLine("Receiving *IDN? response");
            status = ApComm.ap_comm_gets(linkId, vAns, new IntPtr(bufferSize), IntPtr.Zero);
            Console.WriteLine("    done, answer " + Marshal.PtrToStringAnsi(vAns).Trim() + ", status " + status + ".");

            // Set CW frequency
            Console.WriteLine("Setting CW frequency 1.234 GHz");
            status = ApComm.ap_comm_puts(linkId, Marshal.StringToHGlobalAnsi("FREQ 1.234GHZ"), IntPtr.Zero, IntPtr.Zero);
            Console.WriteLine("    done, status " + status + ".");

            // Set CW power
            Console.WriteLine("Setting CW power 3.5 dBm");
            status = ApComm.ap_comm_puts(linkId, Marshal.StringToHGlobalAnsi("POW 3.5"), IntPtr.Zero, IntPtr.Zero);
            Console.WriteLine("    done, status " + status + ".");

            // Set RF output on
            Console.WriteLine("Setting RF output on");
            status = ApComm.ap_comm_puts(linkId, Marshal.StringToHGlobalAnsi("OUTP ON"), IntPtr.Zero, IntPtr.Zero);
            Console.WriteLine("    done, status " + status + ".");

            // Close connection
            Console.WriteLine("Closing connection");
            status = ApComm.ap_comm_close(linkId);
            Console.WriteLine("    done, status " + status + ".");

            Console.ReadLine();
        }
    }
}
